///////////////////////////////////////////////////////////////////////////
// Fast-Approximate Anti Aliasing (FXAA)
///////////////////////////////////////////////////////////////////////////
#include "mta-helper.fx"
//--------------------------------------------------------------------------------------
// Textures
//--------------------------------------------------------------------------------------
texture sTex0 :TEX0;

//--------------------------------------------------------------------------------------
// Sampler Inputs
//--------------------------------------------------------------------------------------

sampler2D InputSampler = sampler_state
{
    Texture = (sTex0);
    MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = POINT;
    AddressU   = Clamp;
	AddressV   = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

struct VSInput
{
    float3 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
};

struct PSInput
{
    float4 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
	float4 interpolatorA : TEXCOORD1;
	float4 interpolatorB : TEXCOORD2;
	float4 interpolatorC : TEXCOORD3;	
};
 
//---------------------------------------------------------------------
//
//---------------------------------------------------------------------
float2 texel = {0.0009765625,0.00130208333333333333333333333333};
float2 fViewportSize = float2(800, 600);
static float2 sPixelSize = float2(1 / fViewportSize.x, 1 / fViewportSize.y);

PSInput vs_fxaa(VSInput VS)
{
    PSInput PS = (PSInput)0;
    PS.Position = mul(float4(VS.Position, 1), gWorldViewProjection);
    PS.TexCoord = VS.TexCoord;

	
	float4 extents;
	float2 offset = sPixelSize;  // ( texel.xy ) * 0.5*1.00;
	extents.xy = VS.TexCoord.xy - offset;
	extents.zw = VS.TexCoord.xy + offset;

	float4 rcpSize;
	rcpSize.xy = -texel.xy * 0.5;
	rcpSize.zw = texel.xy * 0.5*0.32;			
	
	PS.interpolatorA = extents;
	PS.interpolatorB = rcpSize;
	PS.interpolatorC = rcpSize;
	
	PS.interpolatorC.xy *= 4.0;
	PS.interpolatorC.zw *= 4.0*0.60;
		
	
    return PS;
}

float Luminance( float3 c )
{
	return dot( c, float3(0.22, 0.707, 0.071) );
}

#define FxaaTexTop(t, p) tex2Dlod(t, float4(p, 0.0, 0.0))

inline float TexLuminance( float2 uv )
{
	return Luminance(FxaaTexTop(InputSampler, uv).rgb);
}

float3 FxaaPixelShader(float2 pos, float4 extents, float4 rcpSize, float4 rcpSize2)
{
	float lumaNw = TexLuminance(extents.xy);
	float lumaSw = TexLuminance(extents.xw);
	float lumaNe = TexLuminance(extents.zy);
	float lumaSe = TexLuminance(extents.zw);
	
	float3 centre = FxaaTexTop(InputSampler, pos).rgb;
	float lumaCentre = Luminance(centre);
	
	float lumaMaxNwSw = max( lumaNw , lumaSw );
	lumaNe += 0.4/384.0;
	float lumaMinNwSw = min( lumaNw , lumaSw );
	
	float lumaMaxNeSe = max( lumaNe , lumaSe );
	float lumaMinNeSe = min( lumaNe , lumaSe );
	
	float lumaMax = max( lumaMaxNeSe, lumaMaxNwSw );
	float lumaMin = min( lumaMinNeSe, lumaMinNwSw );
	
	float lumaMaxScaled = lumaMax * 0.6*0.015;
	
	float lumaMinCentre = min( lumaMin , lumaCentre );
	float lumaMaxScaledClamped = max( 0.0 , lumaMaxScaled );
	float lumaMaxCentre = max( lumaMax , lumaCentre );
	float dirSWMinusNE = lumaSw - lumaNe;
	float lumaMaxCMinusMinC = lumaMaxCentre - lumaMinCentre;
	float dirSEMinusNW = lumaSe - lumaNw;
	
	if(lumaMaxCMinusMinC < lumaMaxScaledClamped)
		return centre;
	
	float2 dir;
	       dir.x = dirSWMinusNE + dirSEMinusNW;
	       dir.y = dirSWMinusNE - dirSEMinusNW;
	
	dir = normalize(dir);			
	float3 col1 = FxaaTexTop(InputSampler, pos.xy - dir * rcpSize.zw).rgb;
	float3 col2 = FxaaTexTop(InputSampler, pos.xy + dir * rcpSize.zw).rgb;
	
	float dirAbsMinTimesC = min( abs( dir.x ) , abs( dir.y ) ) * 10.0;
	dir = clamp(dir.xy/dirAbsMinTimesC, -5.0, 3.0);
	
	float3 col3 = FxaaTexTop(InputSampler, pos.xy - dir * rcpSize2.zw).rgb;
	float3 col4 = FxaaTexTop(InputSampler, pos.xy + dir * rcpSize2.zw).rgb;
	
	float3 rgbyA = col1 + col2;
	float3 rgbyB = ((col3 + col4) * 0.25) + (rgbyA * 0.25);
	
	if((Luminance(rgbyA) < lumaMin) || (Luminance(rgbyB) > lumaMax))
		return rgbyA * 0.5;
	else
		return rgbyB;
}

//---------------------------------------------------------------------
//
//---------------------------------------------------------------------
float4 ps_fxaa(PSInput PS) : COLOR0
{	
	float3 color = FxaaPixelShader(PS.TexCoord, PS.interpolatorA, PS.interpolatorB, PS.interpolatorC);
	return float4(color, 1.0);
}

//---------------------------------------------------------------------
//
//---------------------------------------------------------------------
technique tec
{
  pass P0
  {
		VertexShader = compile vs_3_0 vs_fxaa();
		PixelShader  = compile ps_3_0 ps_fxaa();

	}

}

/**/